from inspect import signature
from typing import TypeVar, Generic, Callable, Any

from videoag_common.miscellaneous import *
from videoag_common.database import *
from .field import ApiDataField, FieldContext, ApiSimpleColumnField

_O = TypeVar("_O", bound="ApiObject")


class ApiDataMethodField(ApiDataField[_O], Generic[_O]):
    
    # TODO get type id from function return type
    def __init__(self,
                 type_id: str,
                 data_foreign_in_context: bool = False,
                 **kwargs):
        super().__init__(**kwargs)
        self._data_func: Callable[[_O, Any], JsonTypes] or None = None
        self._serializer = None
        self._type_id = type_id
        self._data_type_id = None
        self._data_foreign_in_context = data_foreign_in_context
    
    def data_get_type_id(self) -> str:
        return self._data_type_id
    
    def post_init(self, context: FieldContext):
        super().post_init(context)
        if not callable(context.own_member):
            raise TypeError(f"Member for '{ApiDataMethodField}' must be a callable")
        self._data_func = context.own_member
        
        from ..object import api_get_value_serializer_function
        self._data_type_id, self._serializer = api_get_value_serializer_function(context.all_classes, self._type_id, self._data_foreign_in_context)
    
    def data_get_value(self, object_db: _O, args) -> JsonTypes:
        kwargs = {}
        for para_name, para in signature(self._data_func).parameters.items():
            if para_name == "self":
                continue
            if not hasattr(args, para_name):
                raise ValueError(f"Missing parameter '{para_name}' for serialization of field {self.data_id} in {self.get_api_class().id}")
            kwargs[para_name] = getattr(args, para_name)
        val = self._data_func(object_db, **kwargs)
        return self._serializer(val, args)


class ApiJsonField(ApiSimpleColumnField[_O], Generic[_O]):
    
    def __init__(self, **kwargs):
        super().__init__(**kwargs, type_id="json")
    
    def post_init_check(self, context: FieldContext):
        super().post_init_check(context)
        
        if not isinstance(self._column.type, sql.types.JSON):
            raise TypeError(f"SQL type for json field must be json, but is '{self._column.type}'")
        
        if self.include_in_config:
            raise ValueError("ApiJsonField cannot have include_in_config")
    
    def _db_value_to_json(self, db_value) -> JsonTypes:
        return db_value
    
    def _json_value_to_db(self, session: SessionDb, object_db: _O, json_value: CJsonValue):
        raise AssertionError()
    

