from .course import (
    Course,
    Lecture,
    Chapter,
    LOAD_COURSE_LECTURES,
    LOAD_LECTURE_COURSE,
    LOAD_LECTURE_CHAPTERS,
    LOAD_LECTURE_PUBLISH_MEDIA,
    LOAD_LECTURE_TARGET_MEDIA,
    LOAD_LECTURE_SOURCE_MEDIA,
)
from .site import Announcement, AnnouncementType, AnnouncementPageVisibility, Featured, LOAD_FEATURED_COURSE_OR_LECTURE
from .user import User
from .medium import (
    PublishMedium,
    TargetMedium,
    TargetMediumType,
    PlainVideoTargetMedium,
    PlainAudioTargetMedium,
    ThumbnailTargetMedium,
    SourceMedium,
    SourceMediumStatus,
    MediaProcessTemplate,
    LOAD_SOURCE_MEDIUM_LECTURE,
    LOAD_PUBLISH_MEDIUM_LECTURE,
    LOAD_PUBLISH_MEDIUM_TARGET_MEDIUM,
    LOAD_TARGET_MEDIUM_LECTURE,
    LOAD_TARGET_MEDIUM_PUBLISH_MEDIUM,
    LOAD_TARGET_MEDIUM_PRODUCER_JOB,
)
from .view_permissions import ViewPermissions, ViewPermissionsType, EffectiveViewPermissions
from .changelog import (
    ChangelogEntry,
    ChangelogEntryType,
    ChangelogCreationEntry,
    ChangelogModificationEntry,
    ChangelogDeletionChangeEntry,
    ChangelogUnknownEntry,
    LOAD_CHANGELOG_ENTRY_USER
)
from .job import JobState, Job

from ..api_object import ApiObjectClass, ApiObject, NoSerializationContext


def __init_all_classes() -> dict[str, ApiObjectClass]:
    from ..database import Base
    Base.registry.configure()
    
    all_classes: dict[str, ApiObjectClass] = {}
    for clazz in Base.__subclasses__():
        if not issubclass(clazz, ApiObject):
            _check_no_api_object_subclass(clazz)
            continue
        
        if hasattr(clazz, "__api_class__"):
            api_class = clazz.__api_class__
        else:
            api_class = ApiObjectClass()
            setattr(clazz, "__api_class__", api_class)
        
        api_class.set_class(clazz)
        if api_class.id in all_classes:
            raise ValueError(f"Duplicate API object with id '{api_class.id}'")
        all_classes[api_class.id] = api_class
    
    for object in all_classes.values():
        # noinspection PyProtectedMember
        object._post_init(all_classes)
    
    return all_classes


API_CLASSES_BY_ID = __init_all_classes()
