from datetime import datetime

from videoag_common.database import *
from videoag_common.api_object import *
from .course import Course, responsible_table


class User(ApiObject, Base):
    
    handle: Mapped[str] = api_mapped(
        mapped_column(String(32, STRING_COLLATION), nullable=False),
        ApiStringField(
            include_in_data=True, data_only_mod=True
        )
    )
    name: Mapped[str] = api_mapped(
        mapped_column(Text(collation=STRING_COLLATION), nullable=False),
        ApiStringField(
            include_in_data=True, data_only_mod=True
        )
    )
    
    last_login: Mapped[datetime] = mapped_column(TIMESTAMP(), nullable=True)
    
    enable_mail_notifications: Mapped[bool] = mapped_column(nullable=False, default=True)
    notify_new_video: Mapped[bool] = mapped_column(nullable=False, default=True)
    notify_edit: Mapped[bool] = mapped_column(nullable=False, default=True)
    notify_chapter_suggested: Mapped[bool] = mapped_column(nullable=False, default=True)
    
    responsible_courses: Mapped[list[Course]] = relationship(
        secondary=responsible_table,
        back_populates="responsible_users",
        lazy="raise_on_sql"
    )
